<?php
namespace AIOSEO\Plugin\Common\ImportExport\YoastSeo;

use AIOSEO\Plugin\Common\ImportExport;
use AIOSEO\Plugin\Common\Models;

// phpcs:disable WordPress.Arrays.ArrayDeclarationSpacing.AssociativeArrayFound

/**
 * Imports the post meta from Yoast SEO.
 *
 * @since 4.0.0
 */
class PostMeta {

	/**
	 * Class constructor.
	 *
	 * @since 4.0.0
	 */
	public function scheduleImport() {
		try {
			$actions = as_get_scheduled_actions( [
				'hook'     => aioseo()->importExport->yoastSeo->postActionName,
				'status'   => \ActionScheduler_Store::STATUS_PENDING,
				'per_page' => 1
			]);

			if ( $actions ) {
				return;
			}
			if ( ! get_transient( 'aioseo_import_post_meta_yoast_seo' ) ) {
				set_transient( 'aioseo_import_post_meta_yoast_seo', time(), WEEK_IN_SECONDS );
			}

			\as_schedule_single_action( time(), aioseo()->importExport->yoastSeo->postActionName );
		} catch ( \RuntimeException $e ) {
			// Do nothing.
		}

		$this->importPostMeta();
	}

	/**
	 * Imports the post meta.
	 *
	 * @since 4.0.0
	 *
	 * @return void
	 */
	public function importPostMeta() {
		if ( ! aioseo()->db->tableExists( 'yoast_indexable' ) ) {
			delete_transient( 'aioseo_import_post_meta_yoast_seo' );
			return;
		}

		$postsPerAction  = 100;
		$publicPostTypes = implode( '", "', aioseo()->helpers->getPublicPostTypes( true ) );
		$timeStarted     = gmdate( 'Y-m-d H:i:s', get_transient( 'aioseo_import_post_meta_yoast_seo' ) );

		$posts = aioseo()->db
			->start( 'yoast_indexable' . ' as yi' )
			->select( '*' )
			->leftJoin( 'aioseo_posts as ap', '`yi`.`object_id` = `ap`.`post_id`' )
			->where( 'yi.object_type', 'post' )
			->whereRaw( "( ap.post_id IS NULL OR ap.updated < '$timeStarted' )" )
			->whereRaw( "( yi.object_sub_type IN ( \"$publicPostTypes\" ) )" )
			->limit( $postsPerAction )
			->run()
			->result();

		if ( ! $posts || ! count( $posts ) ) {
			delete_transient( 'aioseo_import_post_meta_yoast_seo' );
			return;
		}

		$mappedMeta = [
			'title'                   => 'title',
			'description'             => 'description',
			'canonical'               => 'canonical_url',
			'is_robots_noindex'       => 'robots_noindex',
			'is_robots_nofollow'      => 'robots_nofollow',
			'is_robots_noarchive'     => 'robots_noarchive',
			'is_robots_noimageindex'  => 'robots_noimageindex',
			'is_robots_nosnippet'     => 'robots_nosnippet',
			'open_graph_title'        => 'og_title',
			'open_graph_description'  => 'og_description',
			'open_graph_image'        => 'og_image_custom_url',
			'open_graph_image_source' => 'og_image_type',
			'open_graph_image_meta'   => '',
			'twitter_title'           => 'twitter_title',
			'twitter_description'     => 'twitter_description',
			'twitter_image'           => 'twitter_image_custom_url',
			'twitter_image_source'    => 'twitter_image_type',
			'schema_page_type'        => 'schema_type',
			'schema_article_type'     => 'schema_type_options'
		];

		foreach ( $posts as $post ) {
			$meta = [
				'post_id' => (int) $post->object_id,
			];

			foreach ( $mappedMeta as $name => $mapping ) {
				if ( empty( $post->$name ) ) {
					continue;
				}

				$value = $post->$name;
				switch ( $name ) {
					case 'is_robots_noindex':
					case 'is_robots_nofollow':
					case 'is_robots_noarchive':
					case 'is_robots_noimageindex':
					case 'is_robots_nosnippet':
						if ( (bool) $value ) {
							$meta[ $mapping ]       = ! empty( $value );
							$meta['robots_default'] = false;
						}
						break;
					case 'open_graph_image':
						$meta['og_image_type'] = 'custom_image';
						$meta[ $mapping ]      = esc_url( $value );
						break;
					case 'twitter_image':
						$meta['twitter_use_og']     = false;
						$meta['twitter_image_type'] = 'custom_image';
						$meta[ $mapping ]           = esc_url( $value );
						break;
					case 'open_graph_image_meta':
						$imageMeta = json_decode( $value );
						if ( ! empty( $imageMeta->width ) && intval( $imageMeta->width ) ) {
							$meta['og_image_width'] = intval( $imageMeta->width );
						}
						if ( ! empty( $imageMeta->height ) && intval( $imageMeta->height ) ) {
							$meta['og_image_height'] = intval( $imageMeta->height );
						}
						break;
					case 'schema_page_type':
						$value = preg_replace( '#\s#', '', $value );
						if ( in_array( $post->object_sub_type, [ 'post', 'page', 'attachment' ], true ) ) {
							break;
						}
						if ( in_array( $value, ImportExport\SearchAppearance::$supportedWebPageGraphs, true ) ) {
							$meta[ $mapping ] = 'WebPage';
							$options          = new \stdClass();
							$options->webPage = [ 'webPageType' => $value ];
						}
						$meta['schema_type_options'] = wp_json_encode( $options );
						break;
					case 'schema_article_type':
						$value = preg_replace( '#\s#', '', $value );
						if ( 'none' === lcfirst( $value ) ) {
							$meta[ $mapping ] = 'None';
							break;
						}

						if ( in_array( $post->object_sub_type, [ 'page', 'attachment' ], true ) ) {
							break;
						}

						$options = new \stdClass();
						if ( isset( $meta['schema_type_options'] ) ) {
							$options = json_decode( $meta['schema_type_options'] );
						}

						$options->article = [ 'articleType' => 'Article' ];
						if ( in_array( $value, ImportExport\SearchAppearance::$supportedArticleGraphs, true ) ) {
							$options->article = [ 'articleType' => $value ];
						} else {
							$options->article = [ 'articleType' => 'BlogPosting' ];
						}

						$meta['schema_type']         = 'Article';
						$meta['schema_type_options'] = wp_json_encode( $options );
						break;
					case 'title':
					case 'description':
					case 'open_graph_title':
					case 'open_graph_description':
						if ( 'page' === $post->object_sub_type ) {
							$value = preg_replace( '#%%primary_category%%#', '', $value );
							$value = preg_replace( '#%%excerpt%%#', '', $value );
						}
						$value = aioseo()->importExport->yoastSeo->helpers->macrosToSmartTags( $value );
					default:
						$meta[ $mapping ] = esc_html( wp_strip_all_tags( strval( $value ) ) );
						break;
				}
			}

			$otherMappedMeta = [
				'_yoast_wpseo_focuskw',
				'_yoast_wpseo_focuskeywords'
			];

			foreach ( $otherMappedMeta as $metaName ) {
				$value = get_post_meta( (int) $post->object_id, $metaName, true );
				if ( ! $value ) {
					continue;
				}

				switch ( $metaName ) {
					case '_yoast_wpseo_focuskw':
						$keyphrase = [
							'focus'      => [ 'keyphrase' => aioseo()->helpers->sanitizeOption( $value ) ],
							'additional' => []
						];
						$meta['keyphrases'] = wp_json_encode( $keyphrase );
						break;
					case '_yoast_wpseo_focuskeywords':
						$keyphrases = [];
						if ( ! empty( $meta[ $mapping ] ) ) {
							$keyphrases = (array) json_decode( $meta[ $mapping ] );
						}

						$yoastKeyphrases = json_decode( $value );
						for ( $i = 0; $i < count( $yoastKeyphrases ); $i++ ) {
							$keyphrase = [ 'keyphrase' => aioseo()->helpers->sanitizeOption( $yoastKeyphrases[ $i ]->keyword ) ];
							$keyphrases['additional'][ $i ] = $keyphrase;
						}
						$meta['keyphrases'] = wp_json_encode( $keyphrases );
						break;
					default:
						break;
				}
			}

			$aioseoPost = Models\Post::getPost( (int) $post->object_id );
			$aioseoPost->set( $meta );
			$aioseoPost->save();
		}

		if ( count( $posts ) === $postsPerAction ) {
			try {
				$actions = as_get_scheduled_actions( [
					'hook'     => aioseo()->importExport->yoastSeo->postActionName,
					'status'   => \ActionScheduler_Store::STATUS_PENDING,
					'per_page' => 1
				]);

				if ( $actions ) {
					return;
				}
				as_schedule_single_action( time(), aioseo()->importExport->yoastSeo->postActionName );
			} catch ( \RuntimeException $e ) {
				// Do nothing.
			}
		} else {
			delete_transient( 'aioseo_import_post_meta_yoast_seo' );
		}
	}
}