<?php
namespace AIOSEO\Plugin\Common\Models;

/**
 * The Post DB Model.
 *
 * @since 4.0.0
 */
class Post extends Model {
	/**
	 * The name of the table in the database, without the prefix.
	 *
	 * @since 4.0.0
	 *
	 * @var string
	 */
	protected $table = 'aioseo_posts';

	/**
	 * Fields that should be json encoded on save and decoded on get.
	 *
	 * @since 4.0.0
	 *
	 * @var array
	 */
	protected $jsonFields = [ 'videos' ];

	/**
	 * Fields that should be hidden when serialized.
	 *
	 * @since 4.0.0
	 *
	 * @var array
	 */
	protected $hidden = [ 'id' ];

	/**
	 * Class constructor.
	 *
	 * @since 4.0.0
	 *
	 * @param mixed $var This can be the primary key of the resource, or it could be an array of data to manufacture a resource without a database query.
	 */
	public function __construct( $var = null ) {
		parent::__construct( $var );

		// Duplicate Post integration.
		add_action( 'dp_duplicate_post', [ $this, 'duplicatePostIntegration' ], 10, 3 );
	}

	/**
	 * Duplicates the model when duplicate post is triggered.
	 *
	 * @since 4.0.0
	 *
	 * @param  integer $newPostId    The new Post ID.
	 * @param  WP_Post $originalPost The original post object.
	 * @param  string  $status       The status of the post.
	 * @return void
	 */
	public function duplicatePostIntegration( $newPostId, $originalPost, $status ) {
		$originalAioseoPost = self::getPost( $originalPost->ID );
		if ( ! $originalAioseoPost->exists() ) {
			return;
		}

		$newPost = self::getPost( $newPostId );
		if ( $newPost->exists() ) {
			return;
		}

		$columns = $originalAioseoPost->getColumns();
		foreach ( $columns as $column => $value ) {
			// Skip the ID columns.
			if ( 'id' === $column || 'post_id' === $column ) {
				continue;
			}

			if ( 'post_id' === $column ) {
				$newPost->$column = $newPostId;
				continue;
			}

			$newPost->$column = $originalAioseoPost->$column;
		}

		$newPost->save();
	}

	/**
	 * Returns a Post with a given ID.
	 *
	 * @since 4.0.0
	 *
	 * @param  int  $postId The Post ID.
	 * @return Post         The Post object.
	 */
	public static function getPost( $postId ) {
		return aioseo()->db
			->start( 'aioseo_posts' )
			->where( 'post_id', $postId )
			->run()
			->model( 'AIOSEO\\Plugin\\Common\\Models\\Post' );
	}

	/**
	 * Get default values for TruSEO page analysis
	 *
	 * @since 4.0.0
	 *
	 * @return object The default TruSEO page analysis values.
	 */
	public static function getPageAnalysisDefaults() {
		$analysisDefaults = [
			'analysis' => [
				'basic'       => [
					'metadescriptionLength' => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 6,
						'title'       => __( 'Meta description length', 'all-in-one-seo-pack' ),
						'description' => __( 'The meta description is too short.', 'all-in-one-seo-pack' )
					],
					'isInternalLink'        => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 3,
						'title'       => __( 'Internal links', 'all-in-one-seo-pack' ),
						'description' => __( 'We couldn\'t find any internal links in your content. Add internal links in your content.', 'all-in-one-seo-pack' )
					],
					'isExternalLink'        => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 3,
						'title'       => __( 'External links', 'all-in-one-seo-pack' ),
						'description' => __( 'No outbound links were found. Link out to external resources.', 'all-in-one-seo-pack' )
					],
					'errors'                => 3
				],
				'title'       => [
					'titleLength'        => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 1,
						'title'       => __( 'Title length', 'all-in-one-seo-pack' ),
						'description' => __( 'No title has been specified. Make sure to write one!', 'all-in-one-seo-pack' )
					],
					'titleHasNumber'     => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 5,
						'title'       => __( 'Title has number', 'all-in-one-seo-pack' ),
						'description' => __( 'Your SEO title doesn\'t contain a number. Add a number to your title to improve CTR.', 'all-in-one-seo-pack' )
					],
					'titleHasPowerWords' => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 5,
						'title'       => __( 'Title has Power Words', 'all-in-one-seo-pack' ),
						'description' => __( 'Your SEO title doesn\'t contain a power word. Add at least one. Power Words are tried-and-true words that copywriters use to attract more clicks.', 'all-in-one-seo-pack' ) // phpcs:ignore Generic.Files.LineLength.MaxExceeded
					],
					'titleSentiment'     => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 5,
						'title'       => __( 'Title Sentiment Words', 'all-in-one-seo-pack' ),
						'description' => __( 'Your SEO title doesn\'t contain a sentiment word. Headlines with a strong emotional sentiment (positive or negative) tend to receive more clicks.', 'all-in-one-seo-pack' ) // phpcs:ignore Generic.Files.LineLength.MaxExceeded
					],
					'errors'             => 4
				],
				'readability' => [
					'contentHasAssets'        => [
						'error'       => 1,
						'maxScore'    => 5,
						'score'       => 0,
						'title'       => __( 'Images/Videos in content', 'all-in-one-seo-pack' ),
						'description' => __( 'You are not using rich media like images or videos.', 'all-in-one-seo-pack' )
					],
					'passiveVoice'            => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 3,
						'title'       => __( 'Passive Voice', 'all-in-one-seo-pack' ),
						'description' => __( 'Use active voice.', 'all-in-one-seo-pack' )
					],
					'subheadingsDistribution' => [
						'error'       => 1,
						'maxScore'    => 9,
						'score'       => 2,
						'title'       => __( 'Subheading distribution', 'all-in-one-seo-pack' ),
						'description' => __( 'You are not using any subheadings, although your text is rather long. Try and add some subheadings.', 'all-in-one-seo-pack' )
					],
					'errors'                  => 3
				]
			]
		];
		return json_decode( wp_json_encode( $analysisDefaults ) );
	}
}